-- Dump a list of JSON objects representing PE 3.3 Dashboard Classifier Groups
WITH

  -- Returns an array of node names associated with each group.
  dashboard_group_nodes AS (
    SELECT
      array_agg(nodes.name) AS nodes,
      node_group_memberships.node_group_id
    FROM
      nodes
    JOIN
      node_group_memberships ON
      nodes.id = node_group_memberships.node_id
    GROUP BY
      node_group_memberships.node_group_id
  ),

  -- Returns an array of tuples crudely expressed using the Postgres "row"
  -- type. This will get cast to JSON as: [{"f1": "key1", "f2": "value1"}, ...]
  dashboard_group_variables AS (
    SELECT
      array_agg(row(key,value)) AS variables,
      parameterable_id
    FROM
      parameters
    WHERE
      parameterable_type = 'NodeGroup'
    GROUP BY
      parameterable_id
  ),

  -- Returns an array of parameter tuples for a specific class.
  dashboard_group_class_parameters AS (
    SELECT
      array_agg(row(key,value)) AS parameters,
      parameterable_id
    FROM
      parameters
    WHERE
      parameterable_type = 'NodeGroupClassMembership'
    GROUP BY
      parameterable_id
  ),

  -- Returns an array of (class name, class parameter) tuples for all classes
  -- associated with a group.
  dashboard_group_classes AS (
    SELECT
      array_agg(row(class.name, class.parameters)) AS "classes",
      class.node_group_id
    FROM (
      SELECT
        node_classes.name,
        COALESCE(dashboard_group_class_parameters.parameters, '{}') as parameters,
        node_group_class_memberships.node_group_id
      FROM
        node_group_class_memberships
      JOIN
        node_classes ON
        node_classes.id = node_group_class_memberships.node_class_id
      -- Use a LEFT OUTER JOIN to pick up classes that were declared without
      -- parameters.
      LEFT OUTER JOIN
        dashboard_group_class_parameters ON
        node_group_class_memberships.id = dashboard_group_class_parameters.parameterable_id
    ) AS "class"
    GROUP BY
      class.node_group_id
  ),

  -- Returns an array of parent group names.
  dashboard_group_parents AS (
    SELECT
      array_agg(node_groups.name) AS parents,
      node_group_edges.from_id
    FROM
      node_groups
    JOIN
      node_group_edges ON
      node_groups.id = node_group_edges.to_id
    GROUP BY
      node_group_edges.from_id
  )

SELECT
  row_to_json(dashboard_groups)
FROM (
  SELECT
    node_groups.name,
    COALESCE(node_groups.description, '') as description,
    COALESCE(dashboard_group_nodes.nodes, '{}') as nodes,
    COALESCE(dashboard_group_variables.variables, '{}') as variables,
    COALESCE(dashboard_group_classes.classes, '{}') as classes,
    COALESCE(dashboard_group_parents.parents, '{}') as parents
  FROM
    node_groups
  LEFT OUTER JOIN
    dashboard_group_nodes ON
    node_groups.id = dashboard_group_nodes.node_group_id
  LEFT OUTER JOIN
    dashboard_group_variables ON
    node_groups.id = dashboard_group_variables.parameterable_id
  LEFT OUTER JOIN
    dashboard_group_classes ON
    node_groups.id = dashboard_group_classes.node_group_id
  LEFT OUTER JOIN
    dashboard_group_parents ON
    node_groups.id = dashboard_group_parents.from_id
) AS dashboard_groups
;
